#include "Arduino.h"
#include <stdio.h>
#include <stdarg.h>
#include <math.h>
#include <Wire.h>
#include <avr/wdt.h>
#include <SoftwareSerial.h>
#include "TimerOne.h"
#include "Print.h"

// Measure the VCC of Arduino and put in the blow macro to get accurate measurements
#define SYSTEM_ACTUAL_VOLTAGE     5.00


// Softserial for Bluetooth
#define BT_TX_PIN    8
#define BT_RX_PIN    7

#define PWM_PIN    9
#define ACS_PIN    A0

#define RUN_PIN         4       // This pin START the ionizer
#define FREQ_SET_PIN    3       // Set different frequency
#define AMPS_SET_PIN    A1


SoftwareSerial bt(BT_RX_PIN, BT_TX_PIN); // RX, TX


// Reset call back
void(* resetCb) (void) = 0;

/* @Variable: freqArray[]
 * Below array is the frequency generator, user can add extra arry element as per frequency reuirements
 * Array elemets are micro seconds to the corrosponding frequency
 * Refer @ commaned code 500 is added extra to get 2KHz Signal
 * */
unsigned int freqArray[] = {20000, 10000, 2000, 1000, 100};  // 50Hz, 100Hz, 500Hz, 1KHz, 10KHz
//unsigned int freqArray[] = {20000, 10000, 2000, 1000, 100, 500};  // 50Hz, 100Hz, 500Hz, 1KHz, 10KHz, 2KHz

volatile bool isEhtRunning = false;

typedef struct ST_SET_VALUES
{
  float currentAmp;
  float setAmp;
  unsigned int freq;
  unsigned int duty;   // In %
}ST_SET_VALUES;

ST_SET_VALUES ehtParameter;

void changeDuty(void)
{
  if(ehtParameter.currentAmp > ehtParameter.setAmp + 0.03)
  {
    ehtParameter.duty--;
    if(ehtParameter.duty < 1)
    {
      ehtParameter.duty = 1;
    }
  }

  else if(ehtParameter.currentAmp < ehtParameter.setAmp - 0.03)
  {
    ehtParameter.duty++;
    if(ehtParameter.duty > 99)
    {
      ehtParameter.duty = 99;
    }
  }
  // Give some time for correct reading of ACS while PWM changes
  delay(20);

  startPwmSignals();
}

void ISR_Handler(void)
{
  ;
}
void initSystem(void)
{
  // Start Serial Connection
  Serial.begin(9600);
  bt.begin(9600);
  Serial.print("WELCOME\n");


  // Uncooment the check code below if user don't want to init system untill BT is connected

  /*while(!bt)  //check if Bluetooth is connected or not
  {
    Serial.print("Finding Bluetooth..\n");
    delay(500);
  } */

  bt.print("WELCOME\n");

  // Init GPIO
  pinMode(RUN_PIN, INPUT_PULLUP);
  pinMode(FREQ_SET_PIN, INPUT_PULLUP);
  pinMode(PWM_PIN, OUTPUT);
  pinMode(10, OUTPUT);
  digitalWrite(PWM_PIN, LOW); // Turn off the PWM Pin

  // Load Default EHT Parameters
  ehtParameter.freq = freqArray[0]; // 50Hz at startup
  ehtParameter.duty = 50;           // 10% Duty at Startup
}

void getAcsReading(void)
{
  unsigned int currentValCount = 0;
  unsigned long readingAvgSum = 0;
  float currentVal = 0.00;

  for(int avg = 0; avg < 100; avg ++)
  {
    currentValCount = analogRead(ACS_PIN);
    readingAvgSum = readingAvgSum + currentValCount;
    delay(2);
  }
  currentValCount = readingAvgSum / 100;
  //currentVal = ((float)currentValCount / 1023.00) * SYSTEM_ACTUAL_VOLTAGE;
  currentVal = ((SYSTEM_ACTUAL_VOLTAGE/ 2.00) - (currentValCount * (SYSTEM_ACTUAL_VOLTAGE / 1024.0)) )/0.185;
  if(currentVal < 0.00)
  {
    currentVal = (currentVal * (-1.00));
  }
  ehtParameter.currentAmp = currentVal;
  readingAvgSum = 0;

}


void changeEhtParam(void)
{
  static int idx;
  unsigned int currentValCount = 0;
  float currentVal = 0.00;
  unsigned long readingAvgSum = 0;

  if(isEhtRunning == false)
  {
    if(digitalRead(FREQ_SET_PIN) == LOW)
    {
      delay(80);
      ehtParameter.freq = freqArray[idx++];
      if(idx > 5)
      {
        idx = 0;
      }
    }
    for(int avg = 0; avg < 100; avg ++)
    {
      currentValCount = analogRead(AMPS_SET_PIN);
      readingAvgSum = readingAvgSum + currentValCount;
      delay(2);
    }
    currentValCount = readingAvgSum / 100;
    currentVal = ((float)currentValCount / 1023.00) * SYSTEM_ACTUAL_VOLTAGE;
    ehtParameter.setAmp = currentVal;
    Timer1.initialize(ehtParameter.freq);
    readingAvgSum = 0;
  }
  else
  {
    getAcsReading();
    changeDuty();
    //Start PWM

  }
}

bool isRunButtonPressed(void)
{
  bool retVal = false;
  if(digitalRead(RUN_PIN) == LOW)
  {
    retVal = true;
    delay(80);
    isEhtRunning = !isEhtRunning;
  }
  return retVal;
}


void startPwmSignals(void)
{
   //Timer1.initialize(ehtParameter.freq);
   Timer1.pwm(PWM_PIN, ((float)ehtParameter.duty / 100) * 1023);

}

void stopPwmSignals(void)
{
  memset(&ehtParameter, 0 , sizeof(ST_SET_VALUES));
  resetCb();  // Call The Reset Callback
}

void setup()
{
  // Clear the ehtParameter buffer variable at startup
  memset(&ehtParameter, 0 , sizeof(ST_SET_VALUES));
  initSystem();

}

// The loop function is called in an endless loop
void loop()
{
  changeEhtParam();
  if(isRunButtonPressed() == true)
  {
    if(isEhtRunning == true)
      startPwmSignals();
    else
      stopPwmSignals();
  }
  Serial.println("================================");
  if(isEhtRunning == true)
    Serial.println("EHT: [RUNNING]");
  else if(isEhtRunning == false)
    Serial.println("EHT: [STOPPED]");

  Serial.print("Duty: "); Serial.println(ehtParameter.duty);
  Serial.print("Freq: "); Serial.print((1.00/(float)ehtParameter.freq) * 1000000); Serial.println(" Hz");
  Serial.print("Set Amps: "); Serial.println(ehtParameter.setAmp);
  Serial.print("ACS Amps: "); Serial.println(ehtParameter.currentAmp);
  Serial.println("================================");

  bt.print("Duty: "); bt.println(ehtParameter.duty);
  bt.print("Freq: "); bt.println(ehtParameter.freq);
  bt.print("Freq: "); bt.print((1.00/(float)ehtParameter.freq) * 1000000); bt.println(" Hz");
  bt.print("ACS Amps: "); bt.println(ehtParameter.currentAmp);
}

